
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# See the License for the specific language governing permissions
# and limitations under the License.
#
#
# Copyright 2011 Grigale Ltd. All rigths reserved.
# Use is sujbect to license terms.
#

__version__ = '1'
__author__ = 'Cyril Plisko <cyril.plisko@grigale.com>'

from .kstat import Kstat

__all__ = ['Kstat']
