import whatap.net as net
import whatap.net.sender as sender

def SendSecure(p):
	sender.Send(net.NET_SECURE_CYPHER, p)

def SendHide(p ):
	sender.Send(net.NET_SECURE_HIDE, p)

def SendNorm(p ) :
	sender.Send(0, p)

