import time

class Initializer(object):
    def init(self):
        pass  # Your implementation for Init goes here


def populateAllTags(whatapTags, tags):
    for k, v in tags.iteritems():
        whatapTags.put_string(k, v)


def populate_all_fields(whatapFields, fields):
    for k, v in fields.iteritems():
        whatapFields.put(k, toValue(v))


def toValue(src):
    if isinstance(src, int):
        return src
    elif isinstance(src, int):
        return int(src)
    elif isinstance(src, int):
        return int(src)
    elif isinstance(src, int):
        return int(src)
    elif isinstance(src, float):
        return float(src)
    elif isinstance(src, float):
        return float(src)
    elif isinstance(src, str):
        return src
    else:
        return str(src)


