class LoadedInput(object):
    def __init__(self, configPath, instance, name):
        self.ConfigPath = configPath
        self.Instance = instance
        self.Name = name


class InputDescriber(object):
    def sample_config(self):
        pass  # Your implementation for SampleConfig goes here


class Input(object):
    def name(self):
        pass  # Your implementation for Name goes here

    def interval(self):
        pass  # Your implementation for Interval goes here

    def gather(self, accumulator):
        pass  # Your implementation for Gather goes here


class ServiceInput(Input):
    def start(self, accumulator):
        pass  # Your implementation for Start goes here

    def stop(self):
        pass  # Your implementation for Stop goes here

    def restart(self):
        pass  # Your implementation for Restart goes here

    def load(self, parseddict):
        pass